#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
=============================================================================
GG - builder for go projects
=============================================================================
http://www.manatlan.com/blog/google_go_language_and_the_gg_builder

released under the terms of BSD License

Copyright (c) 2009, manatlan.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY <copyright holder> ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
=============================================================================

changelog :
    1.2 : bug corrected when building many files
    1.1 : licensed under BSD terms
"""

from subprocess import Popen,PIPE
import string,  sys, re, os

def panic(m):
    print "ERROR:"
    print m
    sys.exit(-1)

try:
    import yaml
except:
    panic("You need pyyaml to run this")

def epath(exe):
    p = os.getenv("GOBIN",None)
    if p:
        return os.path.join(p,exe)
    else:
        return exe

class Bin8:
    g=epath("8g")
    l=epath("8l")
    files = lambda x,l: [os.path.splitext(f)[0]+".8" for f in l ]

class Bin6:
    g=epath("6g")
    l=epath("6l")
    files = lambda x,l: [os.path.splitext(f)[0]+".6" for f in l ]

if os.getenv("GOARCH","386")=="386":
    BIN = Bin8()
else:
    BIN = Bin6()

def run(file):
#    print file
    p = Popen(file,stdout=PIPE,stderr=PIPE)
    out = string.join(p.stdout.readlines() )
    outerr = string.join(p.stderr.readlines() )
    return out  # gobins seems to output errors in stdin ?!? (not in stderr)




def compileEach(l):
    for f in l:
        compileAll([f])

def compileAll(l):
    err=run([BIN.g]+l)
    if err:
        panic(err)

def makepack(name,l):
    err=run( [epath("gopack"),"grc",name] + BIN.files(l) )
    if err:
        panic(err)

def build(name,l):
    err=run([BIN.l,"-o",name] + BIN.files([l[0]]) )
    if err:
        panic(err)

def make(d):
    try:
        exe = []
        d=yaml.load(d.replace("\t","  "))
        for i in [i for i in d.keys() if ".a" in i]:
            l=type(d[i])==list and d[i] or d[i].split(" ")
            compileEach(l)
            makepack(i,l)

        for i in [i for i in d.keys() if ".a" not in i]:
            l=type(d[i])==list and d[i] or d[i].split(" ")
            compileAll(l)
            build(i,l)
            exe.append(i)
        return exe
    except Exception,e:
        panic(str(e))

#~ d="{main: koko.go, kiki.a: kiki.go kiki2.go}"
#~ make(d)
#~ sys.argv=["","koko.go"]

if len(sys.argv)!=2:
    print """USAGE: %s <file>
Build google's go projects by providing a go file or a yaml makefile.
http://www.manatlan.com (C)2009 - V1.2 - BSD License.""" % os.path.basename(sys.argv[0])
    panic("provide one file (.go or makefile)")
else:
    f=sys.argv[1]
    if f.lower().endswith(".go"):
        line1=file(f).readline().strip()
        p=re.search("gg:(\{[^\}]+\})",line1)
        if p:
            doc = p.groups(0)[0]
            print "GG bang project"
        else:
            doc = "{%s: %s}" % (os.path.splitext(f)[0],f)
            print "GG run one"
    else:
        print "GG makefile project"
        try:
            doc = file(f).read()
        except Exception,e:
            panic(str(e))

    l=make(doc)
    if not l:
        print "Just builded"
    elif len(l)==1:
        print "Running '%s':" % l[0]
        if l[0].startswith("/"):
            os.system(l[0])
        else:
            os.system("./"+l[0])
    else:
        print "Multiple executables : ", l
    sys.exit(0)

